'''
A RemindmeRepository is a Repository for storing and retrieving Remindmes.
'''

import sqlite3
from .Remindme import Remindme


class RemindmeRepository:
    '''Repository of Remindmes.'''

    def __init__(self, db_file):
        '''Create a sqlite3 database for remindmes.'''
        self.__db = None
        self.__cursor = None
        self.__remindmes = []
        self.__deleted_remindmes = []
        with sqlite3.connect(db_file) as db:
            self.__db = db
            self.__cursor = db.cursor()
            try:
                sql = 'CREATE TABLE IF NOT EXISTS remindmes(title, content)'
                self.__cursor.execute(sql)
                sql = 'CREATE UNIQUE INDEX IF NOT EXISTS indices ON remindmes(title)'
                self.__cursor.execute(sql)
                self.__db.commit()
                self.__restore_remindmes()
            except Exception:
                raise Exception()

    def __register_remindme(self, remindme):
        self.__remindmes.append(remindme)

    def __restore_remindmes(self):
        '''Restores previously stored remindmes from the database.'''
        try:
            sql = 'SELECT title, content FROM remindmes'
            for item in self.__cursor.execute(sql).fetchall():
                remindme = Remindme(item[0], item[1], repository=self)
                self.__register_remindme(remindme)
        except sqlite3.OperationalError:
            pass
        return self

    def insert_remindme(self, remindme):
        '''Insert remindme into this repository.'''
        try:
            remindme.set_repository(self)
            sql = 'INSERT INTO remindmes VALUES (?,?)'
            self.__cursor.execute(sql, (remindme.get_title(), remindme.get_content(),))
            self.__db.commit()
            self.__register_remindme(remindme)
            return True
        except Exception:
            self.__db.rollback()
            return False

    def create_remindme(self, title, content):
        '''Creates a new remindme in this repository.'''
        remindme = Remindme(title, content, self)
        status = self.insert_remindme(remindme)
        return remindme if status is True else False

    def update_remindme(self, remindme):
        '''Updates a remindme in this repository.'''
        try:
            sql = "UPDATE remindmes SET content='%s' WHERE title='%s' " % (remindme.get_content(), remindme.get_title(),)
            self.__cursor.execute(sql)
            self.__db.commit()
            return True
        except Exception as err:
            self.__db.rollback()
            return False

    def remove_remindme(self, remindme):
        '''Remove remindme from this repository.'''
        if remindme in self.__deleted_remindmes:
            return False
        try:
            sql = 'DELETE FROM remindmes WHERE title == "{0}" '
            sql = sql.format(remindme.get_title())
            self.__cursor.execute(sql)
            self.__db.commit()
            self.__deleted_remindmes.append(remindme)
            self.__filter_out_deleted()
            return True
        except:
            self.__db.rollback()
            return False

    def remove_remindmes(self):
        '''Removes all remindmes from this repository.'''
        for remindme in self.__remindmes:
            status = self.remove_remindme(remindme)
            if status is False:
                return False
        return True

    def __filter_out_deleted(self):
        '''Filters out deleted remindmes.'''
        self.__remindmes = [r for r in self.__remindmes
            if r not in self.__deleted_remindmes]

    def get_remindmes(self):
        '''Return remindmes from database.'''
        return self.__remindmes

    def save_remindmes(self):
        '''Save all remindmes.'''
        for remindme in self.__remindmes:
            remindme.save()

    def find(self, qualify):
        '''Search through the remindmes.'''
        return [x for x in self.__remindmes if qualify(x)]

    def find_by_title(self, title):
        '''Find the remindme by title.'''
        found = self.find(lambda remindme: remindme.get_title() == title)
        return found[0] if found else None

    def count(self):
        return len(self.get_remindmes())



'''
Testing RemindmeRepository model.

Objectives:
 - ensure a consistent interface is maintained
'''

import unittest
from remindme.Remindme import Remindme
from remindme.RemindmeRepository import RemindmeRepository


class Test_RemindmeRepository_Model(unittest.TestCase):
    '''Tests against the RemindmeRepository model
    (remindme.models.RemindmeRepository).'''

    def setUp(self):
        self.db_file = "./test/_test.RemindmeRepository.db"
        self.repository = RemindmeRepository(self.db_file)
        self.data = [
            { "title": "my title is awesome", "content": "and so is my content" },
            { "title": "boring but real title", "content": "get some content for me" },
            { "title": "pizza lover", "content": "some good pizza for me" }
        ]
        for d in self.data:
            self.repository.create_remindme(d["title"], d["content"])

    def test_constructor_restores_remindmes(self):
        num_remindmes = self.repository.count()
        same_repo = RemindmeRepository(self.db_file)
        num_restored_remindmes = same_repo.count()
        self.assertEqual(num_remindmes, num_restored_remindmes,
            '''RemindmeRepository#__init__ fails to restores remindmes''')

    def test_insert_remindme(self):
        title = "gotcha boy"
        content = "pink stars are awesome"
        remindme = Remindme(title, content, self.repository)
        status = self.repository.insert_remindme(remindme)
        self.assertTrue(status,
            '''RemindmeRepository#insert_remindme(remindme) failed to insert
            remindme''')
        found_remindme = self.repository.find_by_title(title)
        self.assertEqual(content, found_remindme.get_content(),
            '''RemindmeRepository#insert_remindme(remindme) failed to insert
            remindme''')

    def test_insert_remindmes_with_same_title(self):
        title = "same old title"
        remindme_1 = Remindme(title, "some content", self.repository)
        remindme_2 = Remindme(title, "other content", self.repository)
        status = self.repository.insert_remindme(remindme_1)
        self.assertTrue(status,
            '''Unexpected error with RemindmeRepository#insert_remindme''')
        status = self.repository.insert_remindme(remindme_2)
        self.assertFalse(status,
            '''RemindmeRepository#insert_remindme fails to return False when
            a remindme fails to be inserted''')

    def test_create_remindme(self):
        title = "some good title, boo"
        content = "got some content for you!"
        remindme = self.repository.create_remindme(title, content)
        self.assertTrue(isinstance(remindme, Remindme),
            '''RemindmeRepository#create_remindme does NOT
            return an instance of Remindme''')

    def test_create_remindmes_with_same_title(self):
        title = "some random content for everyone"
        content = "got some content for you!"
        remindme = self.repository.create_remindme(title, content)
        self.assertTrue(isinstance(remindme, Remindme),
            '''Unexpected error with RemindmeRepository#create_remindme''')
        remindme = self.repository.create_remindme(title, "soem other content")
        self.assertFalse(remindme,
            '''RemindmeRepository#create_remindme fails to return False when a
            remindme with same title is being created''')

    def test_remove_remindme(self):
        remindme_to_remove = self.repository.get_remindmes()[0]
        status = self.repository.remove_remindme(remindme_to_remove)
        self.assertTrue(status,
            '''RemindmeRepository#remove_remindme fails to return True when
            removing a remindme''')
        remindmes = self.repository.get_remindmes()
        found_remindme = (len([r for r in remindmes
            if r.get_title() == remindme_to_remove.get_title()]) == 0)
        self.assertTrue(found_remindme,
            '''RemindmeRepository#remove_remindme fails to remove remindme''')
        status = self.repository.remove_remindme(remindme_to_remove)
        self.assertFalse(status,
            '''RemindmeRepository#remove_remindme fails to return False when
            removing an already removed remindme''')

    def test_remove_remindmes(self):
        remindmes = self.repository.get_remindmes()
        status = self.repository.remove_remindmes()
        num_remindmes = self.repository.count()
        self.assertEqual(0, num_remindmes,
            '''RemindmeRepository#remove_remindmes fails to remove all
            remindmes''')
        self.assertTrue(status,
            '''RemindmeRepository#remove_remindmes fails to return True
            when removing remindes''')
        for r in remindmes:
            self.repository.insert_remindme(r)

    def test_find(self):
        title = "SoME CoMlex! Title tHaT MuST be UniqUE"
        remindme = Remindme(title, "some damn content", self.repository)
        self.repository.insert_remindme(remindme)
        found_remindmes = self.repository.find(lambda r: r.get_title() == title)
        self.assertEqual(1, len(found_remindmes),
            '''RemindmeRepository#find fails to get remindme qualifying a lambda''')
        self.assertEqual(remindme, found_remindmes[0],
            '''RemindmeRepository#find fails to get the remindme''')



'''
A Remindme is an Entity representing a single user-defined remindme.
'''


class Remindme:
    '''A user's single remindme.'''

    def __init__(self, title, content, repository=None):
        '''Creates new remindme.'''
        self.__title = title
        self.__content = content
        self.__repository = repository
        self.__props = {}
        self.__props["saved"] = False
        self.__props["deleted"] = False

    def set_title(self, title):
        '''Set title of this remindme.'''
        self.__title = title
        return self

    def get_title(self):
        '''Return title of this remindme.'''
        return self.__title

    def set_content(self, content):
        '''Set content of this remindme.'''
        self.__content = content
        return self

    def get_content(self):
        '''Return content of this remindme.'''
        return self.__content

    def set_repository(self, repository):
        '''Set repository reference for this remindme.'''
        self.__repository = repository
        return self

    def get_repository(self):
        '''Return repository reference for this remindme.'''
        return self.__repository

    def get_props(self):
        '''Return properties of this remindme.'''
        return self.__props

    def save(self):
        '''Save remindme into repository.'''
        if self.__repository is not None:
            status = self.__repository.insert_remindme(self)
            self.__props["saved"] = status
            return status
        return False

    def set_as_deleted(self):
        self.__props["deleted"] = True

    def delete(self):
        '''Deletes the remindme from repository.'''
        if self.__repository is not None:
            status = self.__repository.remove_remindme(self)
            self.__props["deleted"] = status
            return status
        return False



'''
Testing Remindme model.

Objectives:
 - ensure a consistent interface is maintained
'''

import unittest
from remindme.Remindme import Remindme
from remindme.RemindmeRepository import RemindmeRepository


class Test_Remindme_Model(unittest.TestCase):
    '''Tests against the Remindme model (remindme.models.Remindme).'''

    def setUp(self):
        self.title = "some title"
        self.content = "some content"
        self.repository = RemindmeRepository("./test/_test.Remindme.db")
        self.remindme = Remindme(self.title, self.content, self.repository)

    def test_constructor_sets_title(self):
        self.assertEqual(self.title, self.remindme.get_title(),
            "constructor fails to set title of remindme")

    def test_constructor_sets_content(self):
        self.assertEqual(self.content, self.remindme.get_content(),
            "constructor fails to set content of remindme")

    def test_constructor_sets_repository(self):
        self.assertEqual(self.repository, self.remindme.get_repository(),
            "constructor fails to set repository of remindme")

    def test_title_gets_title(self):
        self.assertEqual(self.title, self.remindme.get_title(),
            "Remindme#get_title() fails to get title of remindme")

    def test_title_sets_title(self):
        self.title = "new title, dummy!"
        self.remindme.set_title(self.title)
        self.assertEqual(self.title, self.remindme.get_title(),
            "Remindme#set_title([title]) fails to set title of remindme")

    def test_content_gets_content(self):
        self.assertEqual(self.content, self.remindme.get_content(),
            "Remindme#get_content() fails to get content of remindme")

    def test_content_sets_content(self):
        self.content = "new content, dummy!"
        self.remindme.set_content(self.content)
        self.assertEqual(self.content, self.remindme.get_content(),
            "Remindme#set_content(content) fails to set content of remindme")

    def test_database_gets_repository(self):
        self.assertEqual(self.repository, self.remindme.get_repository(),
            "Remindme#get_repository() fails to get repository of remindme")

    def test_database_sets_repository(self):
        self.repository = RemindmeRepository("./test/_test.Remindme.new.db")
        self.remindme.set_repository(self.repository)
        self.assertEqual(self.repository, self.remindme.get_repository(),
            "Remindme#set_repository(repository) fails to set repository of remindme")

    def test_props_gets_props(self):
        self.assertTrue(isinstance(self.remindme.get_props(), dict),
            "Remindme#props() gets props not in a dictionary")

    @unittest.skip("not written yet")
    def test_save_saves_into_database(self):
        pass

    @unittest.skip("not written yet")
    def test_delete_deletes_from_database(self):
        pass



import json
import os
import subprocess
import sys
import tempfile
import uuid
from . import config


class Console:
    template = "{color}[{title}]: {message}" + config.COLORS["reset"]

    def __init__(self, title):
        self.title = title

    def raw(self, message):
        sys.stdout.write(message)
        sys.stdout.flush()

    def __write(self, _message, color_code, newline):
        message = Console.template.format(title=self.title,
            color=config.COLORS[color_code], message=_message)
        message += "\n" if newline is True else ""
        self.raw(message)

    def log(self, message, newline=True):
        self.__write(message, "reset", newline)
        return self

    def info(self, message, newline=True):
        self.__write(message, "info", newline)
        return self

    def error(self, message, newline=True):
        self.__write(message, "error", newline)
        return self

    def success(self, message, newline=True):
        self.__write(message, "success", newline)
        return self

    def __input(self):
        if config.PY2:
            return raw_input("> ")
        else:
            return input("> ")

    def get_input(self, question):
        self.info(question + "?")
        return self.__input()

    def get_long_input(self, question):
        self.info(question + "?")
        user_input = []
        while 1:
            try:
                words = self.__input()
                if words == ':end':
                    break
                user_input.append(words)
            except KeyboardInterrupt:
                break
        return '\n'.join(user_input)


class GUI:
    def editor(self, editor, content=None):
        '''Opens an external editor for editing. Returns entered
        contents once editor is closed.

        Throws subprocess.CalledProcessError when executing the editor
        executable.'''
        filename = ".remindme-" + str(uuid.uuid4())
        filepath = os.path.join(tempfile.gettempdir(), filename)
        # if editing old content, add it to the file before opening an
        # editor to edit the file
        if content:
            with open(filepath, "w") as f:
                f.write(content)
        subprocess.check_call([editor, filepath])
        with open(filepath) as f:
            content = f.read()
        os.unlink(filepath)
        return content


class Settings:
    config = {}

    @staticmethod
    def read():
        if os.path.isfile(config.PATHS["config_file"]):
            with open(config.PATHS["config_file"]) as config_file:
                content = config_file.read()
                content = json.loads(content)
                Settings.config = content
        return Settings.config

    @staticmethod
    def write():
        content = json.dumps(Settings.config, sort_keys=True, indent=4)
        with open(config.PATHS["config_file"], "w") as config_file:
            config_file.write(content)



'''
Setup script for RemindMe
'''

from setuptools import setup
import remindme


def get_requirements():
    '''Return a list of requirements for installation as listed in the
    requirements.txt file'''
    with open("requirements.txt", "r") as reqsFile:
      reqs = reqsFile.read()
      return reqs.strip().split("\n")


setup(
    name="remindme",
    version=remindme.__version__,
    author="Gocho Mugo I",
    author_email="mugo@forfuture.co.ke",
    author_url="https://gochomugo.github.io/",
    url="https://github.com/GochoMugo/remindme",
    download_url="https://github.com/GochoMugo/remindme/zipball/master",
    description="Command Line Application for reminding you of something",
    keywords=["remindme", "remind", "remember", "cli"],
    license="MIT",
    long_description=remindme.__doc__,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4"
    ],
    packages=["remindme"],
    install_requires=get_requirements(),
    entry_points={
        'console_scripts': [
            'remindme = remindme.cli:run',
        ]
    }
)
